@@ -2,46 +2,64 @@ import argparse
 import os
 import platform
 import sys
-from typing import List, Optional, Tuple, Union
+import socket
+from typing import List, Optional, Union, Callable
 
 import requests
 from pygments import __version__ as pygments_version
 from requests import __version__ as requests_version
 
-from httpie import __version__ as httpie_version
-from httpie.cli.constants import OUT_REQ_BODY, OUT_REQ_HEAD, OUT_RESP_BODY, OUT_RESP_HEAD
-from httpie.client import collect_messages
-from httpie.context import Environment
-from httpie.downloads import Downloader
-from httpie.output.writer import write_message, write_stream, MESSAGE_SEPARATOR_BYTES
-from httpie.plugins.registry import plugin_manager
-from httpie.status import ExitStatus, http_status_to_exit_status
+from . import __version__ as httpie_version
+from .cli.constants import OUT_REQ_BODY
+from .cli.nested_json import HTTPieSyntaxError
+from .client import collect_messages
+from .context import Environment, Levels
+from .downloads import Downloader
+from .models import (
+    RequestsMessageKind,
+    OutputOptions
+)
+from .output.models import ProcessingOptions
+from .output.writer import write_message, write_stream, write_raw_data, MESSAGE_SEPARATOR_BYTES
+from .plugins.registry import plugin_manager
+from .status import ExitStatus, http_status_to_exit_status
+from .utils import unwrap_context
 
 
 # noinspection PyDefaultArgument
-def main(args: List[Union[str, bytes]] = sys.argv, env=Environment()) -> ExitStatus:
-    """
-    The main function.
-
-    Pre-process args, handle some special types of invocations,
-    and run the main program with error handling.
-
-    Return exit status code.
-
-    """
+def raw_main(
+    parser: argparse.ArgumentParser,
+    main_program: Callable[[argparse.Namespace, Environment], ExitStatus],
+    args: List[Union[str, bytes]] = sys.argv,
+    env: Environment = Environment(),
+    use_default_options: bool = True,
+) -> ExitStatus:
     program_name, *args = args
     env.program_name = os.path.basename(program_name)
     args = decode_raw_args(args, env.stdin_encoding)
-    plugin_manager.load_installed_plugins()
+    plugin_manager.load_installed_plugins(env.config.plugins_dir)
 
-    from httpie.cli.definition import parser
-
-    if env.config.default_options:
+    if use_default_options and env.config.default_options:
         args = env.config.default_options + args
 
     include_debug_info = '--debug' in args
     include_traceback = include_debug_info or '--traceback' in args
 
+    def handle_generic_error(e, annotation=None):
+        msg = str(e)
+        if hasattr(e, 'request'):
+            request = e.request
+            if hasattr(request, 'url'):
+                msg = (
+                    f'{msg} while doing a {request.method}'
+                    f' request to URL: {request.url}'
+                )
+        if annotation:
+            msg += annotation
+        env.log_error(f'{type(e).__name__}: {msg}')
+        if include_traceback:
+            raise
+
     if include_debug_info:
         print_debug_info(env)
         if args == ['--debug']:
@@ -54,6 +72,11 @@ def main(args: List[Union[str, bytes]] = sys.argv, env=Environment()) -> ExitSta
             args=args,
             env=env,
         )
+    except HTTPieSyntaxError as exc:
+        env.stderr.write(str(exc) + "\n")
+        if include_traceback:
+            raise
+        exit_status = ExitStatus.ERROR
     except KeyboardInterrupt:
         env.stderr.write('\n')
         if include_traceback:
@@ -67,7 +90,7 @@ def main(args: List[Union[str, bytes]] = sys.argv, env=Environment()) -> ExitSta
             exit_status = ExitStatus.ERROR
     else:
         try:
-            exit_status = program(
+            exit_status = main_program(
                 args=parsed_args,
                 env=env,
             )
@@ -91,38 +114,50 @@ def main(args: List[Union[str, bytes]] = sys.argv, env=Environment()) -> ExitSta
                 f'Too many redirects'
                 f' (--max-redirects={parsed_args.max_redirects}).'
             )
+        except requests.exceptions.ConnectionError as exc:
+            annotation = None
+            original_exc = unwrap_context(exc)
+            if isinstance(original_exc, socket.gaierror):
+                if original_exc.errno == socket.EAI_AGAIN:
+                    annotation = '\nCouldn’t connect to a DNS server. Please check your connection and try again.'
+                elif original_exc.errno == socket.EAI_NONAME:
+                    annotation = '\nCouldn’t resolve the given hostname. Please check the URL and try again.'
+                propagated_exc = original_exc
+            else:
+                propagated_exc = exc
+
+            handle_generic_error(propagated_exc, annotation=annotation)
+            exit_status = ExitStatus.ERROR
         except Exception as e:
             # TODO: Further distinction between expected and unexpected errors.
-            msg = str(e)
-            if hasattr(e, 'request'):
-                request = e.request
-                if hasattr(request, 'url'):
-                    msg = (
-                        f'{msg} while doing a {request.method}'
-                        f' request to URL: {request.url}'
-                    )
-            env.log_error(f'{type(e).__name__}: {msg}')
-            if include_traceback:
-                raise
+            handle_generic_error(e)
             exit_status = ExitStatus.ERROR
 
     return exit_status
 
 
-def get_output_options(
-    args: argparse.Namespace,
-    message: Union[requests.PreparedRequest, requests.Response]
-) -> Tuple[bool, bool]:
-    return {
-        requests.PreparedRequest: (
-            OUT_REQ_HEAD in args.output_options,
-            OUT_REQ_BODY in args.output_options,
-        ),
-        requests.Response: (
-            OUT_RESP_HEAD in args.output_options,
-            OUT_RESP_BODY in args.output_options,
-        ),
-    }[type(message)]
+def main(
+    args: List[Union[str, bytes]] = sys.argv,
+    env: Environment = Environment()
+) -> ExitStatus:
+    """
+    The main function.
+
+    Pre-process args, handle some special types of invocations,
+    and run the main program with error handling.
+
+    Return exit status code.
+
+    """
+
+    from .cli.definition import parser
+
+    return raw_main(
+        parser=parser,
+        main_program=program,
+        args=args,
+        env=env
+    )
 
 
 def program(args: argparse.Namespace, env: Environment) -> ExitStatus:
@@ -135,6 +170,7 @@ def program(args: argparse.Namespace, env: Environment) -> ExitStatus:
     downloader = None
     initial_request: Optional[requests.PreparedRequest] = None
     final_response: Optional[requests.Response] = None
+    processing_options = ProcessingOptions.from_raw_args(args)
 
     def separate():
         getattr(env.stdout, 'buffer', env.stdout).write(MESSAGE_SEPARATOR_BYTES)
@@ -149,47 +185,54 @@ def program(args: argparse.Namespace, env: Environment) -> ExitStatus:
             and chunk
         )
         if should_pipe_to_stdout:
-            msg = requests.PreparedRequest()
-            msg.is_body_upload_chunk = True
-            msg.body = chunk
-            msg.headers = initial_request.headers
-            write_message(requests_message=msg, env=env, args=args, with_body=True, with_headers=False)
+            return write_raw_data(
+                env,
+                chunk,
+                processing_options=processing_options,
+                headers=initial_request.headers
+            )
 
     try:
         if args.download:
             args.follow = True  # --download implies --follow.
             downloader = Downloader(output_file=args.output_file, progress_file=env.stderr, resume=args.download_resume)
             downloader.pre_request(args.headers)
-        messages = collect_messages(args=args, config_dir=env.config.directory,
+        messages = collect_messages(env, args=args,
                                     request_body_read_callback=request_body_read_callback)
         force_separator = False
         prev_with_body = False
 
         # Process messages as they’re generated
         for message in messages:
-            is_request = isinstance(message, requests.PreparedRequest)
-            with_headers, with_body = get_output_options(args=args, message=message)
-            do_write_body = with_body
-            if prev_with_body and (with_headers or with_body) and (force_separator or not env.stdout_isatty):
+            output_options = OutputOptions.from_message(message, args.output_options)
+
+            do_write_body = output_options.body
+            if prev_with_body and output_options.any() and (force_separator or not env.stdout_isatty):
                 # Separate after a previous message with body, if needed. See test_tokens.py.
                 separate()
             force_separator = False
-            if is_request:
+            if output_options.kind is RequestsMessageKind.REQUEST:
                 if not initial_request:
                     initial_request = message
+                if output_options.body:
                     is_streamed_upload = not isinstance(message.body, (str, bytes))
-                    if with_body:
-                        do_write_body = not is_streamed_upload
-                        force_separator = is_streamed_upload and env.stdout_isatty
+                    do_write_body = not is_streamed_upload
+                    force_separator = is_streamed_upload and env.stdout_isatty
             else:
                 final_response = message
                 if args.check_status or downloader:
                     exit_status = http_status_to_exit_status(http_status=message.status_code, follow=args.follow)
-                    if exit_status != ExitStatus.SUCCESS and (not env.stdout_isatty or args.quiet):
-                        env.log_error(f'HTTP {message.raw.status} {message.raw.reason}', level='warning')
-            write_message(requests_message=message, env=env, args=args, with_headers=with_headers,
-                          with_body=do_write_body)
-            prev_with_body = with_body
+                    if exit_status != ExitStatus.SUCCESS and (not env.stdout_isatty or args.quiet == 1):
+                        env.log_error(f'HTTP {message.raw.status} {message.raw.reason}', level=Levels.WARNING)
+            write_message(
+                requests_message=message,
+                env=env,
+                output_options=output_options._replace(
+                    body=do_write_body
+                ),
+                processing_options=processing_options
+            )
+            prev_with_body = output_options.body
 
         # Cleanup
         if force_separator:
@@ -205,16 +248,15 @@ def program(args: argparse.Namespace, env: Environment) -> ExitStatus:
             if downloader.interrupted:
                 exit_status = ExitStatus.ERROR
                 env.log_error(
-                    'Incomplete download: size=%d; downloaded=%d' % (
-                        downloader.status.total_size,
-                        downloader.status.downloaded
-                    ))
+                    f'Incomplete download: size={downloader.status.total_size};'
+                    f' downloaded={downloader.status.downloaded}'
+                )
         return exit_status
 
     finally:
         if downloader and not downloader.finished:
             downloader.failed()
-        if not isinstance(args, list) and args.output_file and args.output_file_specified:
+        if args.output_file and args.output_file_specified:
             args.output_file.close()
 
 
@@ -228,6 +270,8 @@ def print_debug_info(env: Environment):
     ])
     env.stderr.write('\n\n')
     env.stderr.write(repr(env))
+    env.stderr.write('\n\n')
+    env.stderr.write(repr(plugin_manager))
     env.stderr.write('\n')
 
 
